extends Control

signal square_clicked
signal game_won(winner_idx)
signal game_draw

var active_player_idx = 1

func _ready():
	# connect the pressed-signal of all nine squares
	for row_idx in range(0,3):
		for col_idx in range(0,3):
			# bind the square to the connection so that it will be given as a parameter to the _on_square_selected-function.
			var square = get_square(row_idx, col_idx)
			square.connect("pressed", self, "_on_square_selected", [square])

func clear():
	# call the clear function on every square so we can restart the game
	for row_idx in range(0,3):
		for col_idx in range(0,3):
			var square = get_square(row_idx, col_idx)
			square.clear()

func _on_square_selected(square):
	# when a square is clicked set the owning player
	square.occupy(active_player_idx)
	# tell the rest of the code that a square was clicked
	emit_signal("square_clicked")
	# and check if someone won
	check_win()

func get_square(var row_idx, var col_idx):
	return get_child(row_idx).get_child(col_idx)

func get_square_owner(var row_idx, var col_idx):
	return get_square(row_idx, col_idx).owner

# check all rows, columns and diagonals for a winner
func check_win():
	# use boxing so we can use winner as an out parameter. 
	# Parameters can't be changed, but the thing they are referring to can. 
	# That's why an int as a parameter couldn't change, but an int contained in an object can.
	var winner = Winner.new()
	for idx in range(0,3):
		check_row(idx, winner)
		check_col(idx, winner)
	check_diagonals(winner)
	
	if winner.idx != 0:
		emit_signal("game_won", winner.idx)
	
	if is_full():
		emit_signal("game_draw")


func is_full():
	# search the grid for an empty square
	for row_idx in range(0,3):
		for col_idx in range(0,3):
			var square = get_square(row_idx, col_idx)
			if square.owner == 0:
				# return false, the board is not full since there is still one empty square left
				return false
	
	# if none was found return true, since the board is full
	return true

func check_row(var idx, var out_winner):
	check_squares(idx,0, idx,1, idx,2, out_winner)

func check_col(var idx, var out_winner):
	check_squares(0,idx, 1,idx, 2,idx, out_winner)

func check_diagonals(var out_winner):
	check_squares(0,0, 1,1, 2,2, out_winner)
	check_squares(0,2, 1,1, 2,0, out_winner)

# check if the three squares (x1,y1), (x2,y2) and (x3,y3) are owned by the same player
func check_squares(var x1, var y1, var x2, var y2, var x3, var y3, var out_winner):
	var owner1 = get_square_owner(x1,y1)
	var owner2 = get_square_owner(x2,y2)
	var owner3 = get_square_owner(x3,y3)
	# if all three given squares are owned by the same player and it's not player 0 set the winner index
	if 0 != owner1 and owner1 == owner2 and owner1 == owner3:
		out_winner.idx = owner1

# use a class to box the id of the winner so it can be used as an out parameter.
class Winner:
	var idx = 0
